﻿#################################################
# Author:  Matt Tisdale
# Version: 1.0_08.09.2012
#
# Version: 1.0_08.09.2012
#   Original code creation
#
#################################################

<#
.SYNOPSIS 
	Show replication status of all partitions on all domain controlers in a forest
.PARAMETER Forest
	Name of the forest to connect to. This will default to "dvn.com" if nothing the
    parameter is not used to provide a different forest name.
.EXAMPLE
    Get-dvnADReplicationStatus
    This command will check replication status for all domain controllers in dvn.com
.EXAMPLE
    Get-dvnADReplicationStatus -Forest dvn.dmz
    This command will check replication status for all domain controllers in dvn.dmz
#>



function Get-dvnADReplicationStatus {
    
    param(                        
        [Parameter(Mandatory=$False, HelpMessage="Enter forest name to check:")]
    		[string]
		    $Forest="dvn.com"
    )

    If ($Forest) {
        $forestContext = New-Object System.DirectoryServices.ActiveDirectory.DirectoryContext("Forest", $forest)
        $forestObject = [System.DirectoryServices.ActiveDirectory.Forest]::GetForest($forestContext)
        foreach ($domain in $forestObject.domains) {
            $domainContext = New-Object System.DirectoryServices.ActiveDirectory.DirectoryContext("Domain", $domain)
            $domainObject = [System.DirectoryServices.ActiveDirectory.Domain]::GetDomain($domainContext)
            foreach ($domainController in $domainObject.domainControllers) {
                $dcContext = New-Object System.DirectoryServices.ActiveDirectory.DirectoryContext("DirectoryServer", $domainController)
                $dcObject = [System.DirectoryServices.ActiveDirectory.DomainController]::GetDomainController($dcContext)
                $replicationAttributes = $dcObject.GetAllReplicationNeighbors() | Select PartitionName, SourceServer, LastSuccessfulSync, LastSyncMessage, ConsecutiveFailureCount
                foreach ($i in $replicationAttributes) {
                    $rowData = @{
                        DestinationServer = [string]$domainController
                        PartitionName = [string]$i.PartitionName
                        SourceServer = [string]$i.SourceServer
                        LastSuccessfulSync = [string]$i.LastSuccessfulSync
                        LastSyncMessage = [string]$i.LastSyncMessage
                        ConsecutiveFailureCount = [string]$i.ConsecutiveFailureCount
                    }
                    $rowObject = New-Object -TypeName PSObject -Property $rowData
		            write-output $rowObject
                    $rowData = $null
                    $rowObject = $null
                }
            }
        }
    }
}